#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "bmp.h"

#define NUMBER_OF_COLORS 256

typedef struct {
	BMPHEADER *bh;
	unsigned char *pixelArray;
	unsigned long pixelArraySize;
} MyImage;

void computeHistogram(MyImage *image, double hist[]) {
	unsigned i;
	// Initialize
	for (i = 0; i < NUMBER_OF_COLORS; i++)
		hist[i] = 0.0;
	// Count
	for (i = 0; i < image->pixelArraySize; i++)
		hist[(unsigned)image->pixelArray[i]]++;
	// Normalize
	for (i = 0; i < NUMBER_OF_COLORS; i++)
		hist[i] /= image->pixelArraySize;
}

int segmentByThreshold(MyImage *image, double threshold, const char* fileName) {
	int i;
	unsigned long j;
	double hist[NUMBER_OF_COLORS], sum=0;
	MyImage imgBW; // Not really needed
	unsigned char *bytes;
	BMPHEADER bh;

	bytes = (unsigned char *)malloc(image->pixelArraySize * sizeof(unsigned char));
	imgBW.bh = &bh;
	imgBW.pixelArray = bytes;
	imgBW.pixelArraySize = image->pixelArraySize;
	initBmpStruct(imgBW.bh, image->bh->info->width, image->bh->info->height);

	computeHistogram(image, hist);
	for (i=0; i < NUMBER_OF_COLORS; i++) {
		if (sum >= threshold)
			break;
		sum += hist[i];
	}

	for (j = 0; j < image->pixelArraySize; j++)
		bytes[j] = image->pixelArray[j] < i ? 0: NUMBER_OF_COLORS-1;
	writeImageToFile(fileName, bytes, &bh);

	freeBmpMem(&bh);
	free(bytes);

	return i;
}

double computeDistance(MyImage *image1, MyImage *image2) {
	double diff, dist = 0;
	double hist1[NUMBER_OF_COLORS], hist2[NUMBER_OF_COLORS];
	int i;

	computeHistogram(image1, hist1);
	computeHistogram(image2, hist2);
	for (i = 0; i < NUMBER_OF_COLORS; i++) {
		diff = (hist1[i] - hist2[i]);
		dist += diff * diff;
	}

	return sqrt(dist);
}

int main() {
	unsigned char *bytes, *bytes2, *bytes3;
	int thresh;
	BMPHEADER bh, bh2, bh3;
	MyImage img, img2, img3;

	// Load the image and update the image structure
	loadImage("image.bmp", &bytes, &bh, &img.pixelArraySize);
	img.bh = &bh;
	img.pixelArray = bytes;
	//writeImageToFile("image.bmp", bytes, &bh);

	thresh = segmentByThreshold(&img, 0.2, "imageBW.bmp");
	printf("The threshold color is: %d\n", thresh);

	// Load the second image and update the image structre
	loadImage("barbara.bmp", &bytes2, &bh2, &img2.pixelArraySize);
	img2.bh = &bh2;
	img2.pixelArray = bytes2;
	// Load the second image and update the image structre
	loadImage("bNebula.bmp", &bytes3, &bh3, &img3.pixelArraySize);
	img3.bh = &bh3;
	img3.pixelArray = bytes3;

	printf("The distance between the two images is: %g\n", computeDistance(&img, &img2));
	printf("The distance between the two images is: %g\n", computeDistance(&img, &img3));
	printf("The distance between the two images is: %g\n", computeDistance(&img2, &img3));

	// Free the memory allocated for the second image
	freeBmpMem(&bh2);
	free(bytes2);
	// Free the memory allocated for the second image
	freeBmpMem(&bh3);
	free(bytes3);
	// Free the memory allocated for the first image
	freeBmpMem(&bh);
	free(bytes);

	return 0;
}